@@ -16,8 +16,10 @@ limitations under the License.
 THIS FILE IS DEPRECATED AND MAY BE REMOVED WITHOUT WARNING!
 DO NOT CALL THESE FUNCTIONS IN YOUR CODE!
 """
-import cvxpy.lin_ops.lin_op as lo
+from typing import Tuple
+
 from cvxpy.lin_ops.lin_constraints import LinEqConstr, LinLeqConstr
+import cvxpy.lin_ops.lin_op as lo
 import cvxpy.utilities as u
 import numpy as np
 
@@ -53,7 +55,7 @@ def get_id() -> int:
     return new_id
 
 
-def create_var(shape, var_id=None):
+def create_var(shape: Tuple[int, ...], var_id=None):
     """Creates a new internal variable.
 
     Parameters
@@ -73,7 +75,7 @@ def create_var(shape, var_id=None):
     return lo.LinOp(lo.VARIABLE, shape, [], var_id)
 
 
-def create_param(shape, param_id=None):
+def create_param(shape: Tuple[int, ...], param_id=None):
     """Wraps a parameter.
 
     Parameters
@@ -93,7 +95,7 @@ def create_param(shape, param_id=None):
     return lo.LinOp(lo.PARAM, shape, [], param_id)
 
 
-def create_const(value, shape, sparse=False):
+def create_const(value, shape: Tuple[int, ...], sparse: bool = False):
     """Wraps a constant.
 
     Parameters
@@ -234,7 +236,7 @@ def promote_lin_ops_for_mul(lh_op, rh_op):
     return lh_op, rh_op, shape
 
 
-def mul_expr(lh_op, rh_op, shape):
+def mul_expr(lh_op, rh_op, shape: Tuple[int, ...]):
     """Multiply two linear operators, with the constant on the left.
 
     Parameters
@@ -252,7 +254,7 @@ def mul_expr(lh_op, rh_op, shape):
     return lo.LinOp(lo.MUL, shape, [rh_op], lh_op)
 
 
-def rmul_expr(lh_op, rh_op, shape):
+def rmul_expr(lh_op, rh_op, shape: Tuple[int, ...]):
     """Multiply two linear operators, with the constant on the right.
 
     Parameters
@@ -290,7 +292,7 @@ def multiply(lh_op, rh_op):
     return lo.LinOp(lo.MUL_ELEM, lh_op.shape, [rh_op], lh_op)
 
 
-def kron(lh_op, rh_op, shape):
+def kron(lh_op, rh_op, shape: Tuple[int, ...]):
     """Kronecker product of two matrices.
 
     Parameters
@@ -330,7 +332,7 @@ def div_expr(lh_op, rh_op):
     return lo.LinOp(lo.DIV, lh_op.shape, [lh_op], rh_op)
 
 
-def promote(operator, shape):
+def promote(operator, shape: Tuple[int, ...]):
     """Promotes a scalar operator to the given shape.
 
     Parameters
@@ -348,7 +350,7 @@ def promote(operator, shape):
     return lo.LinOp(lo.PROMOTE, shape, [operator], None)
 
 
-def sum_entries(operator, shape):
+def sum_entries(operator, shape: Tuple[int, ...]):
     """Sum the entries of an operator.
 
     Parameters
@@ -382,7 +384,7 @@ def trace(operator):
     return lo.LinOp(lo.TRACE, (1, 1), [operator], None)
 
 
-def index(operator, shape, keys):
+def index(operator, shape: Tuple[int, ...], keys):
     """Indexes/slices an operator.
 
     Parameters
@@ -402,7 +404,7 @@ def index(operator, shape, keys):
     return lo.LinOp(lo.INDEX, shape, [operator], keys)
 
 
-def conv(lh_op, rh_op, shape):
+def conv(lh_op, rh_op, shape: Tuple[int, ...]):
     """1D discrete convolution of two vectors.
 
     Parameters
@@ -444,7 +446,7 @@ def transpose(operator):
         return lo.LinOp(lo.TRANSPOSE, shape, [operator], None)
 
 
-def reshape(operator, shape):
+def reshape(operator, shape: Tuple[int, ...]):
     """Reshapes an operator.
 
     Parameters
@@ -514,7 +516,7 @@ def upper_tri(operator):
     return lo.LinOp(lo.UPPER_TRI, shape, [operator], None)
 
 
-def hstack(operators, shape):
+def hstack(operators, shape: Tuple[int, ...]):
     """Concatenates operators horizontally.
 
     Parameters
@@ -532,7 +534,7 @@ def hstack(operators, shape):
     return lo.LinOp(lo.HSTACK, shape, operators, None)
 
 
-def vstack(operators, shape):
+def vstack(operators, shape: Tuple[int, ...]):
     """Concatenates operators vertically.
 
     Parameters
